require "envelope_alloc/version"
require "envelope_alloc/gaussian"

module EnvelopeAlloc
  class Provider
    attr_reader :total_num, :alloc_num, :unit, :item_max

    def initialize total_num, alloc_num, options={}
      @total_num = total_num
      @alloc_num = alloc_num
      @unit = options[:unit] || 100
      @item_max = options[:item_max] || (200 * @unit)
    end

    def alloc_items
      return [total_num] if alloc_num == 1

      money_left = (total_num - alloc_num * unit)

      value_vec = []
      0.upto(alloc_num-2) do |i|
        mu = money_left / (alloc_num - i)
        sigma = mu / 2
        rg = RandomGaussian.new(mu, sigma)
        noise_value = rg.rand.to_i
        noise_value = 0 if noise_value < 0
        noise_value = money_left if noise_value > money_left
        threshold_value = item_max - unit
        noise_value = threshold_value if noise_value > threshold_value
        value_vec << (noise_value + unit)
        money_left -= noise_value
      end

      money_left = (total_num - value_vec.reduce(&:+))
      last_money = money_left > item_max ? item_max : money_left
      value_vec << last_money
      money_left = money_left - last_money
      split_left_items(money_left, value_vec)
      value_vec
    end

    def split_left_items(money_left, value_vec)
      return unless money_left > 0
      value_vec.map! do |val|
        gap = item_max - val
        if money_left > gap
          money_left -= gap
          item_max
        else
          value = val + money_left
          money_left = 0
          value
        end
      end
    end
  end
end
